#version 440 core

layout (location = 0) in vec3 aPos;
layout (location = 1) in vec3 aNormal;
layout (location = 2) in vec2 texCoords;

layout(location = 3) in vec3 aTangent;

#define NR_POINT_LIGHTS 1

struct DirLight {
	vec3 direction;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

struct PointLight {
	vec3 position;

	float constant;
	float linear;
	float quadratic;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

struct NormalMapCalc {
	//for normal map calculation
	vec3 TangentLightPos[NR_POINT_LIGHTS];
	vec3 TangentViewPos;
	vec3 TangentFragPos;
	vec3 TangentNormal;
	vec3 TangentLightDiretion;
};

out vec3 Normal;
out vec2 TexCoords;
out vec3 FragPos;

out vec4 EyeSpacePos;
out mat3 TBN_fromTangentTWorld;
out vec3 viewPos_out;
out mat4 modelMatrix_out;

out DirLight dirLight_out;
out PointLight pointLights_out[NR_POINT_LIGHTS];
out NormalMapCalc normalMapCalc_out;

//for vertex calculation
uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

//for light calculation
uniform vec3 viewPos;
uniform DirLight dirLight;
uniform PointLight pointLights[NR_POINT_LIGHTS];


void main()
{

	EyeSpacePos = view*model*vec4(aPos, 1.0);
	
	FragPos = vec3(model * vec4(aPos, 1.0));	
	TexCoords = texCoords;
	Normal = aNormal; //mat3(transpose(inverse(model))) * aNormal;
	modelMatrix_out = model;
	
	//NormalMapping
	mat3 normalMatrix = mat3(transpose(inverse(model)));
	vec3 T = normalize(normalMatrix * aTangent);
	vec3 N = normalize(normalMatrix * aNormal);
	T = normalize(T - dot(T, N) * N);
	vec3 B = normalize(cross(N, T));
	
	mat3 TBN = transpose(mat3(T, B, N));
	TBN_fromTangentTWorld = mat3(T, B, N);

	NormalMapCalc normalMapCalc;

	for (int i = 0; i < NR_POINT_LIGHTS; i++)
	{
		normalMapCalc.TangentLightPos[i] = TBN * pointLights[i].position;
	}
	
	normalMapCalc.TangentViewPos = TBN * viewPos;
	normalMapCalc.TangentFragPos = TBN * FragPos;
	normalMapCalc.TangentNormal = vec3(0, 0, 0); //set in fragment shader
	normalMapCalc.TangentLightDiretion = TBN * dirLight.direction; //set in fragment shader
	normalMapCalc_out = normalMapCalc;

	//lights
	dirLight_out = dirLight;
	pointLights_out = pointLights;
	viewPos_out = viewPos;

    gl_Position = projection * view * model * vec4(aPos, 1.0);

}